import { debounce } from "@/uilts/help";

const keywordData = [
  {
    type: 1,
    keyword: 'giveaway',
    tips: ''
  },
  {
    type: 1,
    keyword: 'follow',
    tips: ''
  },
  {
    type: 2,
    keyword: 'ToolBox',
    tips: ''
  },
  {
    type: 1,
    keyword: 'retweet',
    tips: ''
  },
  {
    type: 2,
    keyword: 'PostEditor',
    tips: ''
  },
]

let tagData = [];
let lastRange;


const getInputBox = () => {
  return document.querySelector('div[data-testid="tweetTextarea_0"]');
}

export const keywordReminderInit = () => {
  listenerKeywordInput();
  addHighlightEleWrapper();
  console.log('keywordReminderInit')
};

export const listenerKeywordInput = debounce(function() {
  let inputBox = getInputBox();
  if(inputBox) {
    inputBox.onkeyup = function () {
        highlightKeyword();
    }
  }
}, 800);

export const highlightKeyword = () => {
  let inputBox = getInputBox();
  if(inputBox) {
    let textEleArr = inputBox.querySelectorAll('span[data-text="true"]');

    if(textEleArr && textEleArr.length) {
      for(let i = 0; i < keywordData.length; i++) {
        let keyword = keywordData[i]['keyword'];
        for(let j = 0; j < textEleArr.length; j++) {
          removeTags({ele: textEleArr[j], rowIndex: j});

          let txt = textEleArr[j]['innerHTML'];
          let startPos = txt.indexOf(keyword), endPos;
          if(startPos > -1) {
            endPos = startPos + keyword.length;
            if(!checkHasTag({keyword, startPos, endPos, rowIndex: j})) {
              addRangeDom({j, keyword, startPos, endPos, eleItem: textEleArr[j]});
            }

            while(startPos > -1) {
              console.log('while ', startPos )
              startPos = txt.indexOf(keyword, startPos + 1);
              endPos = startPos + keyword.length;
              if(startPos > -1) {
                if(!checkHasTag({keyword, startPos, endPos, rowIndex: j})) {
                  addRangeDom({j, keyword, startPos, endPos, eleItem: textEleArr[j], keywordIndex: i});
                }
              }
            }

          }
        }
      }
    } else {
      document.getElementById("denet-keword-highlight-wrapper").innerHTML = '';
      tagData = [];
    }
  }
}

const addRangeDom = (params) => {
  let {j, keyword, startPos, endPos, eleItem, keywordIndex} = params;
  lastRange = window.getSelection().getRangeAt(0)

  let range = new Range();
  range.setStart(eleItem.firstChild, startPos);
  range.setEnd(eleItem.firstChild, endPos);
  window.getSelection().removeAllRanges();
  window.getSelection().addRange(range);

  var range1 = window.getSelection().getRangeAt(0);
  var rectDom = range1.getBoundingClientRect();
  console.log(rectDom);

  window.getSelection().removeAllRanges();

  if(lastRange) {
    window.getSelection().addRange(lastRange)
  }

  tagData.push({
    keyword,
    rowIndex: j,
    startPos,
    endPos
  })

  let inputBox = getInputBox();
  let inputBoxRect = inputBox.getBoundingClientRect();
  let left = rectDom.left - inputBoxRect.left;
  let top = rectDom.top - inputBoxRect.top + 2;

  let cssText = `left: ${left}px;top: ${top}px;width: ${rectDom.width}px;height: ${rectDom.height}px;`;
  addHighlightDom({cssText, rowIndex: j, keyword, keywordIndex, startPos});
}

const checkHasTag = (params) => {
  let hasTag = false;
  for(let i = 0; i < tagData.length; i++) {
    let item = tagData[i];
    if(params.rowIndex == item.rowIndex && params.keyword == item.keyword && item.startPos == params.startPos && item.endPos == params.endPos) {
      hasTag = true;
      break;
    }
  }

  return hasTag;
}

const removeTags = (params) => {
  let {ele} = params;
  for(let i = 0; i < tagData.length; i++) {
    let item = tagData[i];
    let startPos = '', endPos = '';
    if(params.rowIndex == item.rowIndex) {
      if(ele) {
        startPos = ele.innerHTML.indexOf(item.keyword);
        endPos = startPos + item.keyword.length;
        if(startPos > -1) {
          let currentKeyword = ele.innerHTML.substr(item.startPos, item.keyword.length)
          if(currentKeyword != item.keyword) {
            removeTagDom(item);
            tagData.splice(tagData[i], 1);
            i--;
          }
        } else {
          removeTagDom(item);
          tagData.splice(tagData[i], 1);
          i--;
        }
      } else {
        removeTagDom(item);
        tagData.splice(tagData[i], 1);
        i--;
      }
    }
  }
}

const removeTagDom = (params) => {
  let tagDom = document.getElementById(`denet-keyword-tag-${params.rowIndex}-${params.startPos}`);
  if(tagDom) {
    tagDom.remove();
  }
}

export const addHighlightEleWrapper = () => {
  let ele = document.createElement('div');
  ele.style.cssText = 'pointer-events: none;mix-blend-mode: darken;';
  ele.id = 'denet-keword-highlight-wrapper';
  let inputBox = getInputBox();
  if(inputBox) {
    inputBox.parentNode.appendChild(ele);
  }
};

export const addHighlightDom = (params) => {
  let ele = document.createElement('div');
  let domId = `denet-keyword-tag-${params.rowIndex}-${params.startPos}`;
  if(document.getElementById(domId)) {
    return;
  }
  let defaultCssText = 'position: absolute;border: 1.5px dashed #FCAB40;box-sizing: border-box';
  ele.style.cssText = params.cssText + defaultCssText;
  ele.id = domId;
  ele.setAttribute("class", `denet-keyword-tag-${params.rowIndex}`);
  let highlightEleWrapper = document.getElementById('denet-keword-highlight-wrapper');
  if(highlightEleWrapper) {
    highlightEleWrapper.appendChild(ele);
  }
}
